
from pybricks.pupdevices import Motor
from pybricks.parameters import Port, Direction, Stop
from pybricks.tools import wait

motorFlip = Motor(Port.C, Direction.COUNTERCLOCKWISE, [5,27], True)
motorTurn = Motor(Port.A, Direction.COUNTERCLOCKWISE, [[5,27],[12,20]], True)

speed = 100

def init(speed):
    offset = 5

    # move to turn position
    motorFlip.run_until_stalled(-speed, duty_limit=30)
    motorFlip.reset_angle(0)

    # init rotate motor
    motorTurn.run_until_stalled(-speed, duty_limit=30)
    motorTurn.reset_angle(0)
    motorTurn.run_target( speed, 10) # release stress
    motorTurn.reset_angle(0)
    motorTurn.run_target (-speed,  180 + offset)

    # move to flip position
    motorFlip.run_until_stalled( speed, duty_limit=30)

    angleTurn   = 0                    #   0
    angleFlip   = (motorFlip.angle())  # 112
    angleRotate = angleFlip / 2        #  60

    # move to rotate position
    motorFlip.run_target (speed, angleRotate)

    # place cube (needs some code, with the Hub LED)

    return (angleTurn, angleRotate, angleFlip)



def flip(speed, angleTurn, angleRotate, angleFlip):
     
    flipPosition = motorFlip.angle()
    offset = 5

    if flipPosition > angleRotate/2:
        # Rotateposition -> First to turn position
        motorFlip.run_until_stalled(-speed, duty_limit=50)
    
    # move from turn-position to flip position and then back to turn-postition flips the cube.
    motorFlip.run_until_stalled (speed * 5, duty_limit=50)
    motorFlip.run_until_stalled (-speed, duty_limit=50)
    motorFlip.reset_angle(0)

    # move back to initial position (if needed)
    if flipPosition > angleRotate/2: 
        motorFlip.run_target (speed, angleRotate)



def clockwise(speed, angleTurn, angleRotate):

    flipPosition = motorFlip.angle()
    margin, offset = 15, 5
    startAngle = motorTurn.angle()

    if flipPosition > angleRotate/2:
        # Rotate position 

        if   (startAngle > 270 - margin):
            motorTurn.run_target        (-speed,  180)

        elif (startAngle > 180 - margin):       
            motorTurn.run_target        (-speed,   90)

        elif (startAngle >  90 - margin):       
            motorTurn.run_target        (-speed,    0 + offset)

            # relative position reset
            motorTurn.run_until_stalled( -speed/3, duty_limit=30)
            motorTurn.reset_angle(0)
            motorTurn.run_target        ( speed, 10) # release stress
            motorTurn.reset_angle(0)

        elif (startAngle >   0 - margin):       
            motorTurn.run_target        (-speed,  270 + offset)

        else:
            print ('wrong angle')
    else:
        # Turn position 

        if   (startAngle > 270 - margin):
            motorTurn.run_target        (-speed,  180)

        elif (startAngle > 180 - margin):       
            motorTurn.run_target        (-speed,   90)

        elif (startAngle >  90 - margin):       
            motorTurn.run_target        (-speed,    0 + offset)

            # relative position reset
            motorTurn.run_until_stalled( -speed/3, duty_limit=30)
            motorTurn.reset_angle(0)
            motorTurn.run_target        ( speed, 10) # release stress
            motorTurn.reset_angle(0)

        elif (startAngle >   0 - margin):       
            motorTurn.run_target        (-speed,  270 + offset)

        else:
            print ('wrong angle')



def counterclockwise(speed, angleTurn, angleRotate):

    flipPosition = motorFlip.angle()
    margin, offset = 15, 5
    startAngle = motorTurn.angle()

    if flipPosition > angleRotate/2:
        # Rotate position 

        if   (startAngle > 270 - margin):
            motorTurn.run_target        (-speed, 0 + offset)

            # relative position reset
            motorTurn.run_until_stalled( -speed/3, duty_limit=30)
            motorTurn.reset_angle(0)
            motorTurn.run_target        ( speed, 10) # release stress
            motorTurn.reset_angle(0)

        elif (startAngle > 180 - margin):       
            motorTurn.run_target        (-speed,  270 + offset)

        elif (startAngle >  90 - margin):       
            motorTurn.run_target        (-speed,  180 + offset)

        elif (startAngle >   0 - margin):       
            motorTurn.run_target        (-speed,   90 + offset)

        else:
            print ('wrong angle')
    else: 
        # Turn position 

        if   (startAngle > 270 - margin):
            motorTurn.run_target        (-speed, 0 + offset)

            # relative position reset
            motorTurn.run_until_stalled( -speed/3, duty_limit=30)
            motorTurn.reset_angle(0)
            motorTurn.run_target        ( speed, 10) # release stress
            motorTurn.reset_angle(0)

        elif (startAngle > 180 - margin):       
            motorTurn.run_target        (-speed,  270 + offset)

        elif (startAngle >  90 - margin):       
            motorTurn.run_target        (-speed,  180 + offset)

        elif (startAngle >   0 - margin):       
            motorTurn.run_target        (-speed,   90 + offset)

        else:
            print ('wrong angle')



def rotateposition(speed, angleTurn, angleRotate):

    flipPosition = motorFlip.angle()

    if flipPosition > angleRotate/2:
        # already in rotate position 
        pass
    else:
        # turn position
        motorFlip.run_target (speed, angleRotate)



def turnposition(speed, angleTurn, angleRotate):

    flipPosition = motorFlip.angle()

    if flipPosition > angleRotate/2:
        # rotate position 
        motorFlip.run_until_stalled(-speed, duty_limit=30)
        motorFlip.reset_angle(0)
    else:
        # already in turn position
        pass



def command (command, speed, angleTurn, angleRotate, angleFlip):

    moves = {"F":  ['T', 'F', 'CW', 'F' ,'F' ,'F', 'R']  
            ,"F2": ['T', 'F', 'CC', 'CC' ,'F' ,'F' ,'F' , 'R']  
            ,"F'": ['T', 'F', 'CC' ,'F' ,'F' ,'F', 'R']
            ,"B":  ['T', 'F', 'F', 'F', 'CW', 'F', 'R']  
            ,"B2": ['T', 'F', 'F', 'F', 'CC', 'CC', 'F', 'R']
            ,"B'": ['T', 'F', 'F', 'F', 'CC','F', 'R']
            ,"U":  ['T', 'F', 'F', 'CW', 'F', 'F' ,'R']  
            ,"U2": ['T', 'F', 'F', 'CC', 'CC', 'F' ,'F', 'R']
            ,"U'": ['T', 'F', 'F', 'CC', 'F', 'F', 'R']
            ,"D":  ['T', 'CW', 'R']  
            ,"D2": ['T', 'CC', 'CC', 'R']
            ,"D'": ['T', 'CC', 'R']
            ,"L":  ['R', 'CW', 'T', 'F', 'CW', 'F' ,'F' ,'F', 'R', 'CC']
            ,"L2": ['R', 'CW', 'T', 'F', 'CW', 'CW' ,'F' ,'F' ,'F' , 'R', 'CC']
            ,"L'": ['R', 'CW', 'T', 'F', 'CC' ,'F' ,'F' ,'F', 'R', 'CC']
            ,"R":  ['R', 'CC', 'T', 'F', 'CW', 'F' ,'F' ,'F', 'R', 'CW']
            ,"R2": ['R', 'CC', 'T', 'F', 'CW', 'CW' ,'F' ,'F' ,'F', 'R', 'CW']
            ,"R'": ['R', 'CC', 'T', 'F', 'CC' ,'F' ,'F' ,'F', 'R', 'CW']
            }

    movelist = moves [command]
    print (command)
    for move in movelist:
        print ('   ' + move) 
        if move == 'F':
            flip (speed, angleTurn, angleRotate, angleFlip)
        elif move == 'R' :
            rotateposition(speed, angleTurn, angleRotate)
        elif move == 'T' :
            turnposition(speed, angleTurn, angleRotate)
        elif move == 'CC' :
            clockwise(speed, angleTurn, angleRotate)
        elif move == 'CW' :
            counterclockwise(speed, angleTurn, angleRotate)


# initialize the cube with green at the front
# https://rubiks-cube-solver.com
# insert cube with white up

angleTurn, angleRotate, angleFlip = init(speed/3)
clockwise(speed, angleTurn, angleRotate)

wait (5000)

# test each command

# command ("F"  , speed, angleTurn, angleRotate, angleFlip)
# command ("F2" , speed, angleTurn, angleRotate, angleFlip)
# command ("F'" , speed, angleTurn, angleRotate, angleFlip)
# command ("B"  , speed, angleTurn, angleRotate, angleFlip)
# command ("B2" , speed, angleTurn, angleRotate, angleFlip)
# command ("B'" , speed, angleTurn, angleRotate, angleFlip)
# command ("U"  , speed, angleTurn, angleRotate, angleFlip)
# command ("U2" , speed, angleTurn, angleRotate, angleFlip)
# command ("U'" , speed, angleTurn, angleRotate, angleFlip)
# command ("D"  , speed, angleTurn, angleRotate, angleFlip)
# command ("D2" , speed, angleTurn, angleRotate, angleFlip)
# command ("D'" , speed, angleTurn, angleRotate, angleFlip)
# command ("L"  , speed, angleTurn, angleRotate, angleFlip)
# command ("L2" , speed, angleTurn, angleRotate, angleFlip)
# command ("L'" , speed, angleTurn, angleRotate, angleFlip)
# command ("R"  , speed, angleTurn, angleRotate, angleFlip)
# command ("R2" , speed, angleTurn, angleRotate, angleFlip)
# command ("R'" , speed, angleTurn, angleRotate, angleFlip)



'''
# https://rubiks-cube-solver.com/solution.php?cube=0566614351656623331413434525462143153534152252412264621

command ("U", speed, angleTurn, angleRotate, angleFlip)
command ("B2'", speed, angleTurn, angleRotate, angleFlip)
command ("R2", speed, angleTurn, angleRotate, angleFlip)
command ("D", speed, angleTurn, angleRotate, angleFlip)
command ("F'", speed, angleTurn, angleRotate, angleFlip)
command ("D'", speed, angleTurn, angleRotate, angleFlip)
command ("L", speed, angleTurn, angleRotate, angleFlip)
command ("D", speed, angleTurn, angleRotate, angleFlip)
command ("L2", speed, angleTurn, angleRotate, angleFlip)
command ("B2", speed, angleTurn, angleRotate, angleFlip)
command ("U'", speed, angleTurn, angleRotate, angleFlip)
command ("B'", speed, angleTurn, angleRotate, angleFlip)
command ("D", speed, angleTurn, angleRotate, angleFlip)
command ("R2", speed, angleTurn, angleRotate, angleFlip)
command ("U", speed, angleTurn, angleRotate, angleFlip)
command ("B2", speed, angleTurn, angleRotate, angleFlip)
command ("D'", speed, angleTurn, angleRotate, angleFlip)
command ("L2", speed, angleTurn, angleRotate, angleFlip)
command ("U'", speed, angleTurn, angleRotate, angleFlip)
command ("B2", speed, angleTurn, angleRotate, angleFlip)
command ("D'", speed, angleTurn, angleRotate, angleFlip)
command ("R2", speed, angleTurn, angleRotate, angleFlip)
'''

# https://ruwix.com/the-rubiks-cube/rubiks-cube-patterns-algorithms/
# Spiral Pattern

for c in ["L'","B'","D","U","R","U'","R'","D2","R2","D","L","D'","L'","R'","F","U"]:
   command(c,speed,angleTurn,angleRotate,angleFlip)

